/*
 * QosketFactory.cc
 *
 * Copyright (c) 2003 The University of Utah and the Flux Group.
 * All rights reserved.
 *
 * This file is licensed under the terms of the GNU Public License.  
 * See the file "license.terms" for restrictions on redistribution 
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 */

/**
 * @file QosketFactory.cc
 *
 * The shared library interface for libqosket.
 */

#include "config.h"

#include <string.h>
#include <stdarg.h>
#include <assert_pp.h>

#include <iostream>

#include "time_util.h"
#include "HeyParser.hh"
#include "factory_library.h"

#include "quoC.h"

using namespace std;

/**
 * Handle 'hey's directed at a quo::QuoKernel object.
 *
 * @param orb Pointer to the ORB.
 * @param hp The HeyParser that describes the action to take.
 * @return Zero on success, an errno value otherwise.
 */
static int qfKernelHey(CORBA::ORB_ptr orb, HeyParser &hp)
{
    static HeyPropertyInfo suites[] = {
	HeyPropertyInfo("sys-cond",
			(1L << HeyParser::CREATE_PROPERTY),
			"",
			"app-id:string - system condition name.\n"
			"idl-type:string - fully qualified name of the syscond IDL type.\n"
			"impl-type:string - fully qualified name of the implementation class.\n"
			"\n"
			"Create a system condition.\n"),
	HeyPropertyInfo::HPI_NULL
    };
    
    const char *prop_name, *prop_value;
    int retval = EXIT_FAILURE;
    CORBA::Object_var obj;
    quo::QuoKernel_var qk;
    
    obj = orb->string_to_object(hp.who());
    qk = quo::QuoKernel::_narrow(obj.in());
    if( CORBA::is_nil(qk.in()) )
    {
	throw CORBA::BAD_PARAM();
    }
    
    switch( hp.what() )
    {
    case HeyParser::LIST_PROPERTIES:
	try
	{
	    hp.popProperty(prop_name, prop_value);
	}
	catch(const HeyParserException &e)
	{
	    cout << "sys-cond" << endl;
	    retval = EXIT_SUCCESS;
	}
	break;
    case HeyParser::CREATE_PROPERTY:
	hp.popProperty(prop_name, prop_value);
	if( strcasecmp(prop_name, "sys-cond") == 0 )
	{
	    quo::SysCond_var sc;

	    sc = qk->bindSysCond(hp.withValue("app-id"),
				 hp.withValue("idl-type"),
				 hp.withValue("impl-type"));
	    cout << orb->object_to_string(sc.in()) << endl;
	    retval = EXIT_SUCCESS;
	}
	else
	{
	    cerr << "Unknown property: " << prop_name;
	}
	break;
    case HeyParser::GET_SUITES:
	cout << suites;
	retval = EXIT_SUCCESS;
	break;
    default:
	cerr << "Unknown verb..." << endl;
	break;
    }
    return( retval );
}

/**
 * Handle 'hey's directed at a quo::ValueSC object.
 *
 * @param orb Pointer to the ORB.
 * @param hp The HeyParser that describes the action to take.
 * @return Zero on success, an errno value otherwise.
 */
static int qfValueSCHey(CORBA::ORB_ptr orb, HeyParser &hp)
{
    static HeyPropertyInfo suites[] = {
	HeyPropertyInfo("long-value",
			(1L << HeyParser::GET_PROPERTY) |
			(1L << HeyParser::SET_PROPERTY),
			"",
			"Get or set the long value of a ValueSC object.\n"),
	HeyPropertyInfo::HPI_NULL
    };
    
    const char *prop_name, *prop_value;
    int retval = EXIT_FAILURE;
    CORBA::Object_var obj;
    quo::ValueSC_var sc;
    
    obj = orb->string_to_object(hp.who());
    sc = quo::ValueSC::_narrow(obj.in());
    if( CORBA::is_nil(sc.in()) )
    {
	throw CORBA::BAD_PARAM();
    }
    
    switch( hp.what() )
    {
    case HeyParser::LIST_PROPERTIES:
	try
	{
	    hp.popProperty(prop_name, prop_value);
	}
	catch(const HeyParserException &e)
	{
	    cout << "long-value" << endl;
	    retval = EXIT_SUCCESS;
	}
	break;
    case HeyParser::GET_PROPERTY:
	hp.popProperty(prop_name, prop_value);
	if( strcasecmp(prop_name, "long-value") == 0 )
	{
	    cout << sc->longValue() << endl;
	    retval = EXIT_SUCCESS;
	}
	else
	{
	    cerr << "Unknown property: " << prop_name;
	}
	break;
    case HeyParser::SET_PROPERTY:
	hp.popProperty(prop_name, prop_value);
	if( strcasecmp(prop_name, "long-value") == 0 )
	{
	    CORBA::Long l;

	    if( sscanf(hp.to(), "%ld", (long *)&l) == 1 )
	    {
		sc->longValue(l);
		cout << "ok" << endl;
		retval = EXIT_SUCCESS;
	    }
	    else
	    {
		cerr << "Error: not a number" << endl;
	    }
	}
	else
	{
	    cerr << "Unknown property: " << prop_name;
	}
	break;
    case HeyParser::GET_SUITES:
	cout << suites;
	retval = EXIT_SUCCESS;
	break;
    default:
	cerr << "Unknown verb..." << endl;
	break;
    }
    return( retval );
}

int FACTORY_METHOD_SYMBOL(factory_library_op_t op, int tag, va_list args)
{
    int retval = ENOSYS;

    struct {
	const char *hey_server_hint;
	HeyParser *hey_parser;
	CORBA::ORB_ptr orb;
	ostream *out;
	ostream *err;
    } attr = {
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
    };

    while( tag != FMA_TAG_DONE )
    {
	switch( tag )
	{
	case FMA_ORB:
	    attr.orb = va_arg(args, CORBA::ORB_ptr);
	    break;
	case FMA_HeyParser:
	    attr.hey_parser = va_arg(args, HeyParser *);
	    break;
	case FMA_HeyServerHint:
	    attr.hey_server_hint = va_arg(args, const char *);
	    break;
	case FMA_StandardOut:
	    attr.out = va_arg(args, ostream *);
	    break;
	case FMA_StandardError:
	    attr.err = va_arg(args, ostream *);
	    break;
	default:
	    break;
	}
	tag = va_arg(args, int);
    }

    switch( op )
    {
    case FLO_QUERY:
	if( attr.hey_server_hint != NULL )
	{
	    if( strcmp(attr.hey_server_hint, "IDL:quo/QuoKernel:1.0") == 0 )
	    {
		retval = EXIT_SUCCESS;
	    }
	    else if( strcmp(attr.hey_server_hint, "IDL:quo/ValueSC:1.0") == 0 )
	    {
		retval = EXIT_SUCCESS;
	    }
	    else
	    {
	    }
	}
	else
	{
	    cout << "\tIDL:quo/QuoKernel:1.0" << endl
		 << "\tIDL:quo/ValueSC:1.0" << endl;
	}
	break;
    case FLO_HEY:
	if( attr.hey_server_hint != NULL )
	{
	    try
	    {
		if( strcmp(attr.hey_server_hint,
			   "IDL:quo/QuoKernel:1.0") == 0 )
		{
		    retval = qfKernelHey(attr.orb, *attr.hey_parser);
		}
		else if( strcmp(attr.hey_server_hint,
				"IDL:quo/ValueSC:1.0") == 0 )
		{
		    retval = qfValueSCHey(attr.orb, *attr.hey_parser);
		}
		else
		{
		}
	    }
	    catch(const HeyParserException &he)
	    {
		cerr << "Parse error: " << he << endl;
	    }
	    catch(const CORBA::SystemException &e)
	    {
		cerr << "Caught Exception: " << e << endl;
	    }
	    catch(...)
	    {
		cerr << "Caught an unhandled exception" << endl;
	    }
	}
	else
	{
	}
	break;
    default:
	break;
    }
    
    return( retval );
}
