/*
 * LoggingAdvocate.hh
 *
 * Copyright (c) 2003, 2004 The University of Utah and the Flux Group.
 * All rights reserved.
 *
 * This file is licensed under the terms of the GNU Public License.  
 * See the file "license.terms" for restrictions on redistribution 
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 */

/**
 * @file LoggingAdvocate.hh
 *
 * Header file for the LoggingAdvocate class.
 */

#ifndef _logging_advocate_hh
#define _logging_advocate_hh

#include <iostream>

#include <RealTimeTaskDelegateImpl.hh>

/**
 * A decorator that logs the status, advice, period completion, and deadline
 * hit/misses to individual file.
 */
class LoggingAdvocate : public virtual RealTimeTaskDelegateImpl
{

public:

    /** Default constructor. */
    LoggingAdvocate(void);

    /** Destructor. */
    virtual ~LoggingAdvocate(void);

    /**
     * @copydoc BrokerDelegates::Delegate::SetDelegateAttribute
     *
     * Currently supported 'id' values are:
     *
     * @li @e cpu-log The file name where the status value should be logged.
     * @li @e advice-log The file name where the advice value should be logged.
     * @li @e complete-log The file name where the period completion times
     * should be logged.
     * @li @e hit-log The file name where period hits should be logged.
     * @li @e miss-log The file name where period misses should be logged.
     */
    virtual void SetDelegateAttribute(const char *id,
				      const CORBA::Any &value)
	throw (CORBA::SystemException);
    
    /**
     * @copydoc BrokerDelegates::Delegate::GetDelegateAttribute
     *
     * Currently supported 'id' values are:
     *
     * @li @e last-advice The last advice given by the advocate.
     */
    virtual CORBA::Any *GetDelegateAttribute(const char *id)
	throw (CORBA::SystemException);
    
    /** @copydoc Broker::RealTimeTask::PassCPU */
    virtual Broker::CPUReserve  PassCPU(Broker::RealTimeTask_ptr rtt,
					const Broker::CPUReserve &status,
					const Broker::CPUReserve &advice,
					const Broker::KeyedReportParameters &krp)
	throw (CORBA::SystemException);

protected:

    /**
     * The length of the last period in microseconds.
     */
    CORBA::ULong la_LastPeriod;

    /**
     * The time that this object was constructed.
     */
    struct timeval la_StartTime;

    /**
     * The last time a log enty was generated.
     */
    struct timeval la_LastLog;
    
    /**
     * The last time that the CPU usage was logged.
     */
    struct timeval la_LastCPULog;

    /**
     * The output stream for the CPU usage log.
     */
    ofstream *la_CPULog;

    /**
     * The last time the advice was logged.
     */
    struct timeval la_LastAdviceLog;

    /**
     * The output stream for the advice log.
     */
    ofstream *la_AdviceLog;

    /**
     * The output stream for the completion log.
     */
    ofstream *la_CompleteLog;

    /**
     * The output stream for the deadline hit log.
     */
    ofstream *la_HitLog;

    /**
     * The output stream for the deadline miss log.
     */
    ofstream *la_MissLog;

    /**
     * The last advice given by this advocate.
     */
    CORBA::ULong la_LastAdvice;

};

#endif
